/**
 * Copyright (c) 2000-2011 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.portal.freemarker;

import com.liferay.portal.kernel.io.unsync.UnsyncBufferedReader;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.CharPool;
import com.liferay.portal.kernel.util.FileUtil;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;

import java.net.URL;

/**
 * @author Mika Koivisto
 */
public abstract class URLTemplateLoader extends FreeMarkerTemplateLoader {

	public void closeTemplateSource(Object templateSource) {
		if (templateSource instanceof URLTemplateSource) {
			URLTemplateSource urlTemplateSource =
				(URLTemplateSource)templateSource;

			try {
				urlTemplateSource.closeStream();
			}
			catch (IOException ioe) {
			}
		}
	}

	public Object findTemplateSource(String name) throws IOException {
		URL url = getURL(name);

		if (url != null) {
			return new URLTemplateSource(url);
		}

		return null;
	}

	public long getLastModified(Object templateSource) {
		if (templateSource instanceof URLTemplateSource) {
			URLTemplateSource urlTemplateSource =
				(URLTemplateSource)templateSource;

			return urlTemplateSource.getLastModified();
		}

		return super.getLastModified(templateSource);
	}

	public Reader getReader(Object templateSource, String encoding)
		throws IOException {

		if (templateSource instanceof URLTemplateSource) {
			URLTemplateSource urlTemplateSource =
				(URLTemplateSource)templateSource;

			return new UnsyncBufferedReader(
				new InputStreamReader(
					urlTemplateSource.getInputStream(), encoding));
		}

		return null;
	}

	public abstract URL getURL(String name) throws IOException;

	public boolean isTemplateResourceValid(String templateId) {
		if (isBlank(templateId)) {
			return true;
		}

		char[] chars = templateId.toCharArray();

		for (int i = 0; i < chars.length; i++) {
			char c = chars[i];

			if ((c == CharPool.PERCENT) || (c == CharPool.POUND) ||
				(c == CharPool.QUESTION) || (c == CharPool.SEMICOLON)) {

				if (_log.isWarnEnabled()) {
					_log.warn(
						"Unable to load template " + templateId +
							" because the template name contains one or more " +
								"special characters: %, #, ?, or ;");
				}

				return false;
			}

			if (c == CharPool.BACK_SLASH) {
				if (_log.isWarnEnabled()) {
					_log.warn(
						"Unable to load template " + templateId +
							" because the template name contains a backslash");
				}

				return false;
			}
		}

		String extension = FileUtil.getExtension(templateId);

		if (!extension.equals("ftl")) {
			return false;
		}

		return true;
	}

	protected boolean isBlank(String s) {
		if (s == null) {
			return true;
		}

		if (s.length() == 0) {
			return true;
		}

		return false;
	}

	private static Log _log = LogFactoryUtil.getLog(URLTemplateLoader.class);

}