/**
 * Copyright (c) 2000-2011 Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.portal.model.impl;

import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.io.ProtectedObjectInputStream;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.Base64;
import com.liferay.portal.kernel.util.CharPool;
import com.liferay.portal.kernel.util.PropsKeys;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.model.Account;
import com.liferay.portal.model.Company;
import com.liferay.portal.model.CompanyConstants;
import com.liferay.portal.model.Group;
import com.liferay.portal.model.Shard;
import com.liferay.portal.model.User;
import com.liferay.portal.model.impl.CompanyImpl.CompanyKeyObjectInputStream;
import com.liferay.portal.service.AccountLocalServiceUtil;
import com.liferay.portal.service.GroupLocalServiceUtil;
import com.liferay.portal.service.ShardLocalServiceUtil;
import com.liferay.portal.service.UserLocalServiceUtil;
import com.liferay.portal.util.PrefsPropsUtil;
import com.liferay.portal.util.PropsValues;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InvalidClassException;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;

import java.security.Key;
import java.security.KeyRep;

import java.util.Locale;
import java.util.TimeZone;

/**
 * @author Brian Wing Shun Chan
 */
public class CompanyImpl extends CompanyModelImpl implements Company {

	public CompanyImpl() {
	}

	public int compareTo(Company company) {
		String webId1 = getWebId();
		String webId2 = company.getWebId();

		if (webId1.equals(PropsValues.COMPANY_DEFAULT_WEB_ID)) {
			return -1;
		}
		else if (webId2.equals(PropsValues.COMPANY_DEFAULT_WEB_ID)) {
			return 1;
		}
		else {
			return webId1.compareTo(webId2);
		}
	}

	public Account getAccount() throws PortalException, SystemException {
		return AccountLocalServiceUtil.getAccount(
			getCompanyId(), getAccountId());
	}

	public String getAdminName() {
		return "Administrator";
	}

	public String getAuthType() throws SystemException {
		return PrefsPropsUtil.getString(
			getCompanyId(), PropsKeys.COMPANY_SECURITY_AUTH_TYPE,
			PropsValues.COMPANY_SECURITY_AUTH_TYPE);
	}

	public User getDefaultUser() throws PortalException, SystemException {
		return UserLocalServiceUtil.getDefaultUser(getCompanyId());
	}

	public String getDefaultWebId() {
		return PropsValues.COMPANY_DEFAULT_WEB_ID;
	}

	public String getEmailAddress() {

		// Primary email address

		return "admin@" + getMx();
	}

	public Group getGroup() throws PortalException, SystemException {
		if (getCompanyId() > CompanyConstants.SYSTEM) {
			return GroupLocalServiceUtil.getCompanyGroup(getCompanyId());
		}

		return new GroupImpl();
	}

	public Key getKeyObj() {
		if (_keyObj == null) {
			String key = getKey();

			if (_isBlank(key)) {
				return _keyObj;
			}

			try {
				ObjectInputStream ois = new CompanyKeyObjectInputStream(key);

				_keyObj = (Key)ois.readObject();
			}
			catch (Exception e) {
				_log.error(
					"Unable to deserialize " + key +
						", possible Java deserialization attack",
					e);
			}
		}

		return _keyObj;
	}

	public Locale getLocale() throws PortalException, SystemException {
		return getDefaultUser().getLocale();
	}

	public String getName() throws PortalException, SystemException {
		return getAccount().getName();
	}

	public String getShardName() throws PortalException, SystemException {
		Shard shard = ShardLocalServiceUtil.getShard(
			Company.class.getName(), getCompanyId());

		return shard.getName();
	}

	public String getShortName() throws PortalException, SystemException {
		return getName();
	}

	public TimeZone getTimeZone() throws PortalException, SystemException {
		return getDefaultUser().getTimeZone();
	}

	public boolean hasCompanyMx(String emailAddress)
		throws SystemException {

		emailAddress = emailAddress.trim().toLowerCase();

		int pos = emailAddress.indexOf(CharPool.AT);

		if (pos == -1) {
			return false;
		}

		String mx = emailAddress.substring(pos + 1, emailAddress.length());

		if (mx.equals(getMx())) {
			return true;
		}

		String[] mailHostNames = PrefsPropsUtil.getStringArray(
			getCompanyId(), PropsKeys.ADMIN_MAIL_HOST_NAMES,
			StringPool.NEW_LINE, PropsValues.ADMIN_MAIL_HOST_NAMES);

		for (int i = 0; i < mailHostNames.length; i++) {
			if (mx.equalsIgnoreCase(mailHostNames[i])) {
				return true;
			}
		}

		return false;
	}

	public boolean isAutoLogin() throws SystemException {
		return PrefsPropsUtil.getBoolean(
			getCompanyId(), PropsKeys.COMPANY_SECURITY_AUTO_LOGIN,
			PropsValues.COMPANY_SECURITY_AUTO_LOGIN);
	}

	public boolean isCommunityLogo() throws SystemException {
		return PrefsPropsUtil.getBoolean(
			getCompanyId(), PropsKeys.COMPANY_SECURITY_COMMUNITY_LOGO,
			PropsValues.COMPANY_SECURITY_COMMUNITY_LOGO);
	}

	public boolean isSendPassword() throws SystemException {
		return PrefsPropsUtil.getBoolean(
			getCompanyId(), PropsKeys.COMPANY_SECURITY_SEND_PASSWORD,
			PropsValues.COMPANY_SECURITY_SEND_PASSWORD);
	}

	public boolean isSendPasswordResetLink() throws SystemException {
		return PrefsPropsUtil.getBoolean(
			getCompanyId(), PropsKeys.COMPANY_SECURITY_SEND_PASSWORD_RESET_LINK,
			PropsValues.COMPANY_SECURITY_SEND_PASSWORD_RESET_LINK);
	}

	public boolean isStrangers() throws SystemException {
		return PrefsPropsUtil.getBoolean(
			getCompanyId(), PropsKeys.COMPANY_SECURITY_STRANGERS,
			PropsValues.COMPANY_SECURITY_STRANGERS);
	}

	public boolean isStrangersVerify() throws SystemException {
		return PrefsPropsUtil.getBoolean(
			getCompanyId(), PropsKeys.COMPANY_SECURITY_STRANGERS_VERIFY,
			PropsValues.COMPANY_SECURITY_STRANGERS_VERIFY);
	}

	public boolean isStrangersWithMx() throws SystemException {
		return PrefsPropsUtil.getBoolean(
			getCompanyId(), PropsKeys.COMPANY_SECURITY_STRANGERS_WITH_MX,
			PropsValues.COMPANY_SECURITY_STRANGERS_WITH_MX);
	}

	public void setKey(String key) {
		_keyObj = null;

		super.setKey(key);
	}

	public void setKeyObj(Key keyObj) {
		_keyObj = keyObj;

		super.setKey(Base64.objectToString(keyObj));
	}

	private boolean _isBlank(String s) {
		if (s == null) {
			return true;
		}

		if (s.length() == 0) {
			return true;
		}

		return false;
	}

	private Key _keyObj = null;

	private static Log _log = LogFactoryUtil.getLog(CompanyImpl.class);

	class CompanyKeyObjectInputStream extends ProtectedObjectInputStream {

		public CompanyKeyObjectInputStream(String key) throws IOException {
			super(new ByteArrayInputStream(Base64.decode(key)));
		}

		@Override
		protected Class<?> doResolveClass(ObjectStreamClass objectStreamClass)
			throws ClassNotFoundException, IOException {

			Class streamClass = super.doResolveClass(objectStreamClass);

			if (streamClass == null) {
				return null;
			}

			if (_log.isDebugEnabled()) {
				_log.debug("Deserializating " + streamClass);
			}

			if (streamClass.isArray() &&
				streamClass.getComponentType().isPrimitive()) {

				return streamClass;
			}

			if (Key.class.isAssignableFrom(streamClass)) {
				return streamClass;
			}

			if (KeyRep.class.isAssignableFrom(streamClass)) {
				return streamClass;
			}

			if (Enum.class.isAssignableFrom(streamClass)) {
				return streamClass;
			}

			throw new InvalidClassException(
				"Rejected resolving of illegal class " +
					objectStreamClass.getName());
		}
	}

}